import 'package:flx_ui/flx_ui.dart';
import 'package:get/get.dart';

typedef TargetTapCallback = void Function();
typedef TargetLongTapCallback = void Function();
typedef TargetDoubleTapCallback = void Function();

class Target extends StatelessWidget {
  final double borderRadius;

  final Widget child;

  final TargetTapCallback onTap;
  final TargetLongTapCallback? onLongTap;
  // final TargetDoubleTapCallback? onDoubleTap;

  final bool clipped;

  const Target({
    Key? key,
    required this.child,
    required this.onTap,
    this.borderRadius = Metrics.borderRadius,
    this.clipped = true,
    this.onLongTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return _buildAll();
  }

  Widget _buildAll() {
    return GetBuilder<TargetController>(
      init: TargetController(),
      global: false,
      builder: (controller) => _buildGestures(
        controller: controller,
        child: _buildBackground(
          controller: controller,
          child: clipped
              ? ClipRRect(
                  borderRadius: BorderRadius.circular(borderRadius),
                  child: _buildSplashAnimation(
                    controller: controller,
                    child: child,
                  ),
                )
              : _buildSplashAnimation(
                  controller: controller,
                  child: child,
                ),
        ),
      ),
    );
  }

  Widget _buildGestures({
    required Widget child,
    required TargetController controller,
  }) {
    return GestureDetector(
      onTap: () => onTap(),
      onTapDown: (event) {
        controller.position = event.localPosition;
        controller.onTap();
      },
      onLongPress: onLongTap ?? () {},
      onLongPressStart: (event) => controller.onLongStart(event),
      onLongPressMoveUpdate: (event) => controller.onLongMove(event),
      onLongPressEnd: (event) => controller.onLongEnd(event),
      child: MouseRegion(
        onEnter: (event) => controller.onEnter(event),
        onHover: (event) => controller.onHover(event),
        onExit: (event) => controller.onExit(event),
        cursor: SystemMouseCursors.click,
        child: child,
      ),
    );
  }

  Widget _buildBackground({
    required Widget child,
    required TargetController controller,
  }) {
    return Container(
      decoration: BoxDecoration(
        color: controller.isHover ? Colors.white.withOpacity(0.05) : null,
        borderRadius: BorderRadius.circular(borderRadius),
      ),
      child: child,
    );
  }

  Widget _buildSplashAnimation({
    required Widget child,
    required TargetController controller,
  }) {
    return AnimatedBuilder(
      animation: controller.opacityAnimController,
      builder: (context, child) => AnimatedBuilder(
        animation: controller.radiusAnimController,
        builder: (context, child) => CustomPaint(
          foregroundPainter: TargetWavePainter(
            radius: controller.radiusAnim.value,
            opacity: controller.opacityAnim.value,
            color: Colors.white,
            position: controller.position,
          ),
          child: child,
        ),
        child: child,
      ),
      child: child,
    );
  }
}
