import 'package:flx_ui/flx_ui.dart';
import 'package:get/get.dart';

typedef ToggleCallback = void Function(bool value);

class Toggle extends StatelessWidget {
  final ToggleCallback? onChanged;
  final bool isEnable;

  const Toggle({
    Key? key,
    this.onChanged,
    this.isEnable = false,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GetBuilder<ToggleController>(
      init: ToggleController(isEnable),
      global: false,
      builder: (controller) => FlatCard(
        width: 65,
        borderRadius: 50,
        clipped: true,
        child: Target(
          borderRadius: 0,
          clipped: false,
          onTap: () {
            controller.onTap();
            if (onChanged != null) {
              onChanged!(controller.isEnable);
            }
          },
          child: AnimatedAlign(
            alignment: controller.isEnable
                ? Alignment.centerRight
                : Alignment.centerLeft,
            duration: Duration(milliseconds: 225),
            curve: Curves.easeInOutCirc,
            child: Padding(
              padding: const EdgeInsets.symmetric(vertical: 4, horizontal: 4),
              child: DeepCard(
                borderRadius: 12,
                color: controller.isEnable
                    ? ColorTheme.accent(context)
                    : Colors.grey,
                width: 22,
                height: 22,
                child: SizedBox(),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
